import 'isomorphic-fetch';
import 'form-data';
import URLSearchParams from 'url-search-params';
import { isObject } from './misc';

const noop = () => { };

function prepend(prefix, name, separator) {
  if (prefix) {
    if (separator)
      return prefix + "." + name;

    return prefix + name;
  }
  return name;
}

function flatten(prefix, obj, map) {
  if (obj || obj === 0) {
    if (Array.isArray(obj)) {
      for (let i = 0; i < obj.length; i++) {
        var item = obj[i];
        flatten(prepend(prefix, "[" + i + "]", false), item, map);
      }
    } else if (isObject(obj)) {
      if (obj['__type__'] === 'map') {
        for (let key in obj) {
          if (key !== '__type__')
            flatten(prepend(prefix, "[" + key + "]", false), obj[key], map);
        }
      } else if (obj instanceof File) {
        // deal with JS based file load
        map[prefix] = obj;
      } else {
        for (let key in obj) {
          flatten(prepend(prefix, key, true), obj[key], map);
        }
      }
    }
    else {
      map[prefix] = obj;
    }
  }
}

export function toFlattenMap(obj) {
  let map = {};

  flatten(null, obj, map);
  return map;
}

export function composeFetch(service, endpoint, data, headers) {
  let fullUrl = !!service ? service + endpoint : endpoint;

  let method = !!data && !!data.method ? data.method : 'POST';

  if (method === 'POST') {
    let params = new URLSearchParams();
    if (!!data && !!data.commandObject) {
      let flattenMap = toFlattenMap(data.commandObject);
      for (let key in flattenMap) {
        params.append(key, flattenMap[key]);
      }
    }
    return ({
      fullUrl,
      fetchData: {
        headers,
        credentials: 'include',
        method,
        body: params
      }
    });

  } else {
    let params = [];
    if (!!data && !!data.commandObject) {
      let flattenMap = toFlattenMap(data.commandObject);

      let i = 0;
      for (let key in flattenMap) {
        if (i++ === 0)
          params.push('?');
        else
          params.push('&');

        params.push(encodeURIComponent(key) + '=' + encodeURIComponent(flattenMap[key]));
      }

      if (params.length > 0)
        fullUrl = fullUrl + params.join('');
    }

    return ({
      fullUrl,
      fetchData: {
        credentials: 'include',
        method
      }
    });
  }
}

export function callApi(service, endpoint, data) {
  if (typeof service !== 'object') {
    const fetchInfo = composeFetch(service, endpoint, data);
    return new Promise((resolve, reject) => {
      fetch(fetchInfo.fullUrl, fetchInfo.fetchData)
        .then((response) => {
          return response.text().then(text => text);
        })
        .then((response) => {
          const data = JSON.parse(response.replace(/([^\\])":(\d{15,})/g, '$1":"$2"'));
          if (Number(data.errorCode) !== 200) {
            return reject(data);
          } else {
            return resolve(data);
          }
        })
        .catch((err) => {
          console.error(err);
        });
    });
  } else {
    const { service: prefix = '/evh', api, data: commandObject = {}, method, success = noop, error, promise = false } = service;
    const fetchInfo = composeFetch(prefix, api, { commandObject, method });

    return new Promise((resolve, reject) => {
      fetch(fetchInfo.fullUrl, fetchInfo.fetchData)
        .then((response) => {
          return response.text().then(text => text);
        })
        .then((response) => {
          const data = JSON.parse(response.replace(/([^\\])":(\d{15,})/g, '$1":"$2"'));
          if (promise) {
            if (Number(data.errorCode) === 200) {
              return resolve(data.response);
            } else {
              return reject(data);
            }
          } else {
            if (Number(data.errorCode) === 200) {
              success(data.response);
            } else {
              if (error)
                error(data);
            }
          }
        })
        .catch((err) => {
          console.error(err);
        });
    });
  }
}

export function callRaw(service, endpoint, data, headers) {
  let fetchInfo = composeFetch(service, endpoint, data, headers);
  return fetch(fetchInfo.fullUrl, fetchInfo.fetchData);
}
